@@ -1,8 +1,13 @@
 {-# LANGUAGE BangPatterns #-}
-module Network.HTTP.Date.Converter (epochTimeToHTTPDate) where
+module Network.HTTP.Date.Converter ( epochTimeToHTTPDate
+                                   , httpDateToUTC
+                                   , utcToHTTPDate
+                                   ) where
 
 import Control.Applicative
 import Data.ByteString.Internal
+import Data.Time
+import Data.Time.Calendar.WeekDate
 import Data.Word
 import Foreign.Marshal.Array
 import Foreign.Ptr
@@ -35,6 +40,40 @@ epochTimeToHTTPDate x = defaultHTTPDate {
     (y,m,d) = toYYMMDD days
     (h,n,s) = toHHMMSS secs
 
+-- | Translating 'HTTPDate' to 'UTCTime'.
+--
+--   Since 0.0.7.
+httpDateToUTC :: HTTPDate -> UTCTime
+httpDateToUTC x = UTCTime (fromGregorian y m d) (secondsToDiffTime s)
+  where
+    y = fromIntegral $ hdYear x
+    m = hdMonth x
+    d = hdDay x
+    s = fromIntegral $ (hdHour   x `rem` 24) * 3600
+                     + (hdMinute x `rem` 60) * 60
+                     + (hdSecond x `rem` 60)
+
+-- | Translating 'UTCTime' to 'HTTPDate'.
+--
+--   Since 0.0.7.
+utcToHTTPDate :: UTCTime -> HTTPDate
+utcToHTTPDate x = defaultHTTPDate {
+    hdYear   = fromIntegral y
+  , hdMonth  = m
+  , hdDay    = d
+  , hdHour   = h
+  , hdMinute = n
+  , hdSecond = round s
+  , hdWkday  = fromEnum (w :: Int)
+  }
+  where
+    (y, m, d) = toGregorian day
+    (h, n, s) = ((todHour tod), (todMin tod), (todSec tod))
+    (_, _, w) = toWeekDate day
+    day       = localDay time
+    tod       = localTimeOfDay time
+    time      = utcToLocalTime utc x
+
 toYYMMDD :: Int -> (Int,Int,Int)
 toYYMMDD x = (yy, mm, dd)
   where
